(in-package "USER" :use '("KR" "KR-DEBUG" "GARNET-DEBUG"))

(defvar INIT
  (progn
    (garnet-load "gadgets:motif-menubar-loader")
    (garnet-load "gadgets:standard-edit")
;    (garnet-load "gadgets:scrolling-menu-loader")
;    (garnet-load "gadgets:motif-scrolling-menu-loader")
    ))

(when (boundp 'WIN) (opal:destroy WIN))

(create-instance 'WIN inter:interactor-window
  (:left 700) (:top 100)
  (:width 200) (:height 200)
  (:background-color opal:motif-gray))
(s-value WIN :aggregate (create-instance 'TOP-AGG opal:aggregate))
;(opal:update win)


(create-instance 'M gg:motif-menubar
  (:items
   `(("File" NIL (("Open..." Show-Read-Dialog)
		  ("New" New-Func)
		  ("Save As..." Show-Save-Dialog)))
     ("Edit" NIL (("Cut" gg:standard-cut)
		  ("Copy" gg:standard-copy)
		  ("Paste" gg:standard-paste-inc-place)))
     ("Control" NIL (("Value Control" Value-Control-Func)
		     ("Enable Control" Enable-Control-Func)
		     ("Other Control..." Standard-NIY)
		     )))))

(opal:add-components TOP-AGG M)
(opal:update WIN)

