(in-package "USER" :use '("LISP"))

(Defparameter Garnet-Lapidary-Files
  '(
	"extras"
	"parameters"
	"lapidary"
	"support-misc"
	"support-halftones"
	"support-selection"		"selection"
	"create-object"
	"delete-object"
	"move-grow"
	"attach-constraints"

	"support-menus"
	"support-3d-menus"
	"support-menu-properties"	"menu-properties"
	"support-menu-constraints"	"menu-constraints"
					"menu-constraints-position"
					"menu-constraints-dimension"
	"support-menu-editor"		"menu-editor"
        "gadgetcopy"
        "gadgetio" "save-restore"))

(dolist (file Garnet-Lapidary-Files)
	 (compile-file (merge-pathnames file 
				#+cmu "lapidary:"
				#+(not cmu) Garnet-Lapidary-PathName
				)
		       ))
