(setf load-lapidary-p t)
;(setf Garnet-KR-PathName "/afs/cs/user/dzg/garnet/kr/new/")
;(setf Garnet-Opal-PathName "/afs/cs/user/ecp/garnet/opal/new/")
;(setf Garnet-Inter-PathName "/afs/cs/user/bam/garnet/inter/new/")
;(setf Garnet-Aggregadgets-PathName "/afs/cs/user/ecp/garnet/aggregadgets/new/")
;(setf Garnet-Lapidary-PathName "/afs/cs/user/bvz/garnet/lapidary/")
;(setf Garnet-Lapidary-PathName "/usr/bvz/garnet/lapidary/")

;(setf (ext:search-list "lapidary:")
;      (list Garnet-Lapidary-PathName))
